VERSION 1.0 CLASS
BEGIN
  MultiUse = -1  'True
  Persistable = 0  'NotPersistable
  DataBindingBehavior = 0  'vbNone
  DataSourceBehavior  = 0  'vbNone
  MTSTransactionMode  = 0  'NotAnMTSObject
END
Attribute VB_Name = "cBitmap"
Attribute VB_GlobalNameSpace = False
Attribute VB_Creatable = True
Attribute VB_PredeclaredId = False
Attribute VB_Exposed = False
Option Explicit

' GDI+ Wrapper written by Paul
' http://www.syix.com/wpsjr1/index.html
'
' Wrapper will Dispose on setting equal to Nothing (terminate event)
' And you are responsible for Disposing before any new bitmap is created
' Failure to Dispose properly will lead to various problems/crashes.
'
' Currently, functions that make a new bitmap are:
' CreateFromScan0, CreateFromHBITMAP and LoadFromFile

Private Declare Sub OleCreatePictureIndirect Lib "oleaut32.dll" (lpPictDesc As PICDESC, riid As CLSID, ByVal fOwn As Long, ByRef lplpvObj As Any)

Public gdip As cGDIP ' Reference for Error Handling

Private bitmap          As Long
Private lCurErr         As Long
Private lPixelFormat    As Long
Private m_lFrameCount   As Long
Private m_lWidth        As Long
Private m_lHeight       As Long
Private m_lNumCodecs    As Long
Private rc              As RECT
Private tGuids()        As CLSID ' array of GUIDs for codecs
Private colCodecs       As Collection   ' search into codec index on file extension
Private Stream          As IUnknown
Private bProperties()   As Byte
Private pi              As PROPERTYITEM
Private m_lPropSize     As Long
Private b()             As Byte

Public Function CreateFromScan0(ByVal lWidth As Long, ByVal lHeight As Long, ByVal lStride As Long, ByVal ePixelFormats As PixelFormats, ByRef lDataPtr As Long)
  CreateFromScan0 = Execute(GdipCreateBitmapFromScan0(lWidth, lHeight, lStride, ePixelFormats, ByVal lDataPtr, bitmap))
  GetPixelFormat
  GetDimension
End Function

Public Function CreateFromHBitmap(ByVal hBitmap As Long) As Long
  CreateFromHBitmap = Execute(GdipCreateBitmapFromHBITMAP(hBitmap, 0, bitmap))
  GetPixelFormat
  GetDimension
End Function

Public Property Get CurErr() As Long
  CurErr = lCurErr
End Property

Public Sub Dispose()
  If bitmap Then GdipDisposeImage bitmap
  Set Stream = Nothing
End Sub

Private Function EnumEncoders() As Long
  Dim lNumEncoders  As Long
  Dim lEncoderSize  As Long
  Dim lError        As Long
  Dim b()           As Byte
  Dim i             As Long
  Dim codecs()      As ImageCodecInfo
  
  lError = GdipGetImageEncodersSize(lNumEncoders, lEncoderSize)
  If lError = 0 Then
    ReDim codecs(lNumEncoders - 1)
    ReDim b(lEncoderSize - 1)
    
    lError = GdipGetImageEncoders(lNumEncoders, lEncoderSize, b(0))
    If lError = 0 Then
      RtlMoveMemory codecs(0), b(0), lNumEncoders * LenB(codecs(0))
      ReDim tGuids(lNumEncoders - 1)
      m_lNumCodecs = lNumEncoders
      Set colCodecs = Nothing
      Set colCodecs = New Collection
      
      Do While lNumEncoders
        lNumEncoders = lNumEncoders - 1
        tGuids(lNumEncoders) = codecs(lNumEncoders).CLSID
        ParseOnChar StringFromPointerW(codecs(lNumEncoders).pwszFilenameExtension), ";", lNumEncoders
      Loop
    Else
      Error lError
    End If
  Else
    Error lError
  End If
End Function

Friend Property Get EncoderGuid(ByVal lIndex As Long) As CLSID
  EncoderGuid = tGuids(lIndex)
End Property

Private Function Execute(ByVal lReturn As Long) As Long
  If lReturn Then
    lCurErr = lReturn
    If Not gdip Is Nothing Then gdip.Error lReturn, "cBitmap"
  Else
    lCurErr = 0
  End If
  Execute = lCurErr
End Function

' do not compare this to a boolean
'  returns -1 for not found, 0-positive GUID index for found
Public Function ExtensionExists(ByRef sKey As String) As Long
  On Error GoTo errorHandler
  ExtensionExists = True ' invalid index
  
  If Not colCodecs Is Nothing Then
    ExtensionExists = colCodecs.Item(sKey)
  End If
  
  Exit Function
errorHandler:
  ' exit silently
End Function

Public Function LoadFromFile(ByVal sFilename As String) As Long
  LoadFromFile = Execute(GdipLoadImageFromFile(sFilename, bitmap))
  GetPixelFormat
  GetDimension
End Function


Private Sub GetDimension()
  Dim sngWidth  As Single
  Dim sngHeight As Single
  
  Execute GdipGetImageDimension(bitmap, sngWidth, sngHeight)
  m_lWidth = sngWidth
  m_lHeight = sngHeight
  rc.Right = m_lWidth
  rc.Bottom = m_lHeight
End Sub


Private Sub GetPixelFormat()
  Execute GdipGetImagePixelFormat(bitmap, lPixelFormat)
End Sub


Public Property Get Height() As Long
  Height = m_lHeight
End Property


Public Function GetPropertyItem(ByVal eTag As PropertyTags, ByRef vValue As Variant, ByVal lPropertyIndex As Long) As Long
  Dim lSize As Long
  
  GetPropertyItem = Execute(GdipGetPropertyItemSize(bitmap, eTag, lSize))
  If GetPropertyItem = Status.Ok Then
    ReDim bProperties(lSize - 1)
    GetPropertyItem = Execute(GdipGetPropertyItem(bitmap, eTag, lSize, bProperties(0)))
    If GetPropertyItem = Status.Ok Then
      RtlMoveMemory pi, bProperties(0), LenB(pi) ' 2 bytes padding in memory
      If pi.Value And pi.length > 0 Then
        ReDim bProperties(pi.length - 1)
        RtlMoveMemory bProperties(0), ByVal pi.Value, pi.length
        m_lPropSize = pi.length
        If pi.Type = PropertyTagTypeByte Or pi.Type = PropertyTagTypeShort Or pi.Type = PropertyTagTypeLong Then
          vValue = GetPropertyNumeric(pi.Type, lPropertyIndex)
        ElseIf pi.Type = PropertyTagTypeASCII Or pi.Type = PropertyTagTypeRational Then
          'vValue = GetPropertyString(pi.Type, lPropertyIndex)
        Else
          Debug.Print "Error - Unhandled PropertyTagType"
        End If
      End If
    End If
  End If
End Function

Private Function GetPropertyNumeric(ByVal eTagtype As PropertyTagTypes, ByVal lPropIndex As Long) As Long
  Dim lSize As Long
  Dim lIndex As Long
  
    Select Case eTagtype
      Case PropertyTagTypeByte
        lSize = 1
      
      Case PropertyTagTypeShort
        lSize = 2
      
      Case PropertyTagTypeLong
        lSize = 4
    End Select
    
    lIndex = lPropIndex * lSize
    If lIndex >= 0 And lIndex < m_lPropSize Then
      RtlMoveMemory GetPropertyNumeric, bProperties(lIndex), lSize
    End If
End Function

Friend Function GetPixelARGB(ByVal X As Long, ByVal Y As Long) As ARGB
  Execute GdipBitmapGetPixel(bitmap, X, Y, GetPixelARGB)
End Function

Public Function GetPixelLong(ByVal X As Long, ByVal Y As Long) As Long
  Execute GdipBitmapGetPixel(bitmap, X, Y, GetPixelLong)
End Function

Friend Function SetGdiPlusPixel(ByVal X As Long, ByVal Y As Long, ByVal higcolor As Long)
   SetGdiPlusPixel = Execute(GdipBitmapSetPixel(bitmap, X, Y, ByVal higcolor))
End Function

Friend Function GetGdiPlusPixel(ByVal X As Long, ByVal Y As Long, higcolor As ARGB) As Long
   GetGdiPlusPixel = Execute(GdipBitmapGetPixel(bitmap, X, Y, higcolor))
End Function


Friend Function LockBits(ByRef rc As RECT, ByVal eMode As ImageLockMode, ByVal eFormat As PixelFormats, ByRef bd As BitmapData) As Long
  LockBits = Execute(GdipBitmapLockBits(bitmap, rc, eMode, eFormat, bd))
End Function

Public Property Get PixelFormat() As PixelFormats
  PixelFormat = lPixelFormat
End Property

Friend Property Get Rectangle() As RECT
  Rectangle = rc
End Property

Friend Function SaveToFile(ByVal sFilename As String, ByRef tEncoder As CLSID, ByVal lEncParamPointer As Long) As Long
  SaveToFile = Execute(GdipSaveImageToFile(bitmap, sFilename, tEncoder, ByVal lEncParamPointer))
End Function


Private Function StringFromPointerW(ByVal lPointer As Long) As String
  Dim lLength As Long
  
  If lPointer Then
    lLength = lstrlenW(lPointer)
    StringFromPointerW = Space$(lLength)
    RtlMoveMemory ByVal StrPtr(StringFromPointerW), ByVal lPointer, lLength * 2
  End If
End Function

Friend Function StringToGuid(ByRef sGuid As String) As CLSID
  CLSIDFromString sGuid, StringToGuid
End Function

Private Sub ParseOnChar(ByRef sIn As String, ByRef sChar As String, ByVal lGuidIndex As Long)
  Dim lStartPosition As Long
  Dim lFoundPosition As Long
  Dim sItem          As String
  
  lFoundPosition = InStr(sIn, sChar)
  lStartPosition = 1
  
  Do While lFoundPosition
    sItem = Mid$(sIn, lStartPosition, lFoundPosition - lStartPosition)
    colCodecs.Add lGuidIndex, sItem
    lStartPosition = lFoundPosition + 1
    lFoundPosition = InStr(lStartPosition, sIn, sChar)
  Loop
  
  sItem = Trim$(Mid$(sIn, lStartPosition))
  If LenB(sItem) Then colCodecs.Add lGuidIndex, sItem
End Sub


Friend Function UnlockBits(ByRef bd As BitmapData) As Long
  UnlockBits = Execute(GdipBitmapUnlockBits(bitmap, bd))
End Function

Public Property Get Width() As Long
  Width = m_lWidth
End Property

Private Sub Class_Initialize()
  EnumEncoders
End Sub

Private Sub Class_Terminate()
  Dispose
End Sub

